#!/usr/bin/env python
# -*- coding: utf-8 -*-

# py_tools_ds
#
# Copyright (C) 2019  Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
#
# This software was developed within the context of the GeoMultiSens project funded
# by the German Federal Ministry of Education and Research
# (project grant code: 01 IS 14 010 A-C).
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
test_topology
-------------

Tests for `py_tools_ds.geo.vector.topology` module.
"""

from unittest import TestCase, main
from shapely.geometry import Polygon, MultiPolygon, Point

from py_tools_ds.geo.vector.topology import get_overlap_polygon, fill_holes_within_poly


class Test_get_overlap_polygon(TestCase):
    def setUp(self):
        self.polygon_1 = Polygon([(0, 0), (0, 5), (5, 5), (5, 0)])
        self.polygon_2 = Polygon([(1, 0), (1, 5), (6, 5), (6, 0)])
        self.polygon_3 = Polygon([(-1, 0), (-1, -5), (-6, -5), (-6, 0)])

    def test_with_overlap(self):
        out = get_overlap_polygon(self.polygon_1, self.polygon_2)
        self.assertIsInstance(out, dict)

        self.assertTrue(out['overlap poly'].equals(Polygon([(1, 0), (1, 5), (5, 5), (5, 0)])))
        self.assertEqual(out['overlap percentage'], 80)
        self.assertEqual(out['overlap area'], 20)

    def test_no_overlap(self):
        out = get_overlap_polygon(self.polygon_1, self.polygon_3)
        self.assertIsInstance(out, dict)

        self.assertTrue(out['overlap poly'] is None)
        self.assertEqual(out['overlap percentage'], 0)
        self.assertEqual(out['overlap area'], 0)

    # TODO test the case where the output polygon is a multipolygon


class Test_fill_holes_within_poly(TestCase):
    def setUp(self):
        self.polygon_1 = Polygon([(0, 0), (5, 0), (5, 5), (0, 5)])  # large poly
        self.polygon_2 = Polygon([(1, 1), (1, 2), (2, 2), (2, 1)])  # small poly within large poly
        self.polygon_3 = Polygon([(3, 3), (3, 4), (5, 4), (4, 3)])  # small poly within large poly
        self.multipolygon_1 = MultiPolygon([self.polygon_1, self.polygon_2])
        self.multipolygon_2 = MultiPolygon([self.polygon_2, self.polygon_3])

    def test_polygon(self):
        filled_poly = fill_holes_within_poly(self.polygon_1)
        self.assertIsInstance(filled_poly, Polygon)
        self.assertTrue(filled_poly == self.polygon_1)

    def test_multipolygon_with_smaller_poly_inside(self):
        filled_poly = fill_holes_within_poly(self.multipolygon_1)
        self.assertIsInstance(filled_poly, Polygon)
        self.assertTrue(filled_poly == self.polygon_1)

    def test_multipolygon_is_disjunct(self):
        with self.assertWarns(RuntimeWarning):
            filled_poly = fill_holes_within_poly(self.multipolygon_2)

        self.assertIsInstance(filled_poly, Polygon)
        self.assertTrue(filled_poly == self.polygon_3)

    def test_empty_multipolygon(self):
        with self.assertRaises(ValueError):
            fill_holes_within_poly(MultiPolygon())

    def test_wrong_input(self):
        with self.assertRaises(ValueError):
            # noinspection PyTypeChecker
            fill_holes_within_poly(Point((0, 1)))


if __name__ == '__main__':
    main()
